@@ -23,20 +23,37 @@
 #ifndef PKG_NODE_H
 #define PKG_NODE_H
 
-#include "vs_treeitem.h"
+#include <vscreen/vs_treeitem.h>
+#include <generic/undo.h>
+
+class undo_group;
+class keybindings;
 
 class pkg_tree_node:virtual public vs_treeitem
 // Provides some extra package-related interfaces.
 {
 public:
-  virtual void select()=0;
-  virtual void hold()=0;
-  virtual void remove()=0;
-  virtual void purge()=0;
+  virtual void select(undo_group *undo)=0;
+  virtual void hold(undo_group *undo)=0;
+  virtual void remove(undo_group *undo)=0;
+  virtual void purge(undo_group *undo)=0;
+  virtual void reinstall(undo_group *unfo)=0;
+  // set_auto is idempotent!  No stupid toggling stuff.
+  virtual void set_auto(bool isauto, undo_group *undo)=0;
+  // Any undo information needed should be added to the passed-in undo group
 
-  bool dispatch_char(chtype ch);
+  bool dispatch_char(chtype ch, vs_tree *owner);
   // IMPORTANT NOTE: pkg_tree_node::dispatch_char() does NOT call
   // vs_treeitem::dispatch_char!
+
+  static keybindings *bindings;
+  static void init_bindings();
+};
+
+class apt_undo_group:public undo_group
+{
+public:
+  void undo();
 };
 
 #endif
